# Copyright (c) 2011 Cloudera, Inc. All rights reserved.

import os
import shutil
try:
  from cStringIO import StringIO
except:
  from StringIO import StringIO

from cmf.monitor import MonitorDaemon, schema, constants
from cmf.monitor.activitymonitor import ActivityMonitorMonitor, json_parser
from cmf.monitor.tests import make_config_file

from nose.tools import assert_true, assert_equals, assert_false

class MockActivityMonitorMonitor(ActivityMonitorMonitor):
  def __init__(self, daemon, registry, string):
    ActivityMonitorMonitor.__init__(self, daemon, registry)
    self.set_string(string)
    self.set_throw_error(None)

  def set_string(self, string):
    self._string = string

  def set_throw_error(self, error):
    self._error = error

  def urlopen(self, url):
    if self._error:
      raise self._error
    return StringIO(self._string)

# CM 4.0 config file format.
def _make_cm40_format_config_file(role_name):
  properties = """[activitymonitor]
role_name = %s
service_name = my_mgmt1
host = localhost
firehose_debug_port = 8086
collect_interval = 1.0
"""
  return  make_config_file(properties % (role_name,))

# CM 4.1 config file: we added the log directory.
def _make_cm41_format_config_file(role_name):
  properties = """[activitymonitor]
role_name = %s
service_name = my_mgmt1
host = localhost
firehose_debug_port = 8086
log_dir = /path/to/logs
collect_interval = 1.0
"""
  return  make_config_file(properties % (role_name,))

def test_activitymonitor_lifecycle():
  do_activitymonitor_lifecycle(_make_cm40_format_config_file, False)
  do_activitymonitor_lifecycle(_make_cm41_format_config_file, True)

def do_activitymonitor_lifecycle(make_config_file_function,
                                   expect_directory_metrics):
  daemon = MonitorDaemon("host", "host", agent=None)
  reg = daemon.get_registry_for_testing()
  monitor = MockActivityMonitorMonitor(daemon, reg, json_parser.SAMPLE)
  monitor.refresh_pid(os.getpid(), True)

  # Make two different configs
  conf_dir = make_config_file_function("mgmt-AMON-test")
  conf_dir2 = make_config_file_function("mgmt-AMON")

  try:
    assert_equals(0, len(reg.get_registry()))

    # Start it
    monitor.refresh(conf_dir)
    assert_equals(monitor, reg.get_registry().pop())
    monitor.collect()
    monitor.clear_last_reported_timestamp()
    role_update = monitor.get_role_update().to_avro_dict()
    assert_equals('mgmt-AMON-test', role_update['rolename'])

    # Refresh with a different config
    monitor.refresh(conf_dir2)
    assert_equals(monitor, reg.get_registry().pop())
    assert_equals(1, len(reg.get_registry()))

    monitor.collect()

    # Test metrics and role update
    monitor.clear_last_reported_timestamp()
    role_update = monitor.get_role_update().to_avro_dict()

    assert_equals('mgmt-AMON', role_update['rolename'])
    assert_equals('my_mgmt1', role_update['service_name'])
    metric_dict = { }
    for metric_obj in role_update['metrics']:
      metric_dict[metric_obj['id']] = metric_obj['value']
    assert_true(metric_dict[schema.TOTAL_CPU_USER] >= 0.0)
    assert_true(metric_dict[schema.TOTAL_CPU_SYSTEM] >= 0.0) 
    assert_equals(5, metric_dict[schema.AMON_ACTIVITY_TREE_DROPS]) 
    assert_equals(48, metric_dict[schema.AMON_ACTIVITY_TREE_PROCESSED]) 
    assert_equals(1, metric_dict[schema.AMON_ACTIVITY_TREE_QUEUE_SIZE]) 
    assert_equals(10, metric_dict[schema.AMON_ACTIVITY_MONITOR_DROPS]) 
    assert_equals(48, metric_dict[schema.AMON_ACTIVITY_MONITOR_PROCESSED]) 
    assert_equals(20, metric_dict[schema.AMON_ACTIVITY_MONITOR_QUEUE_SIZE]) 
    assert_equals(constants.WEB_METRICS_COLLECTION_GOOD,
                  metric_dict[schema.WEB_METRICS_COLLECTION_STATUS])
    assert_true(schema.WEB_METRICS_COLLECTION_DURATION in metric_dict)
    assert_equals(5, metric_dict[schema.AMON_PURGER_SERVICE_RUNS])
    assert_equals(3, metric_dict[schema.AMON_PURGER_SERVICE_SLOW_RUNS])
    assert_equals(1, metric_dict[schema.AMON_PURGER_SERVICE_RUNS_WITH_EXCEPTION])

    dir_metrics = role_update['dirUpdates']
    if expect_directory_metrics:
      assert_true(len(dir_metrics) == 1, "We configured 1 dirs: %s" % dir_metrics)
      assert_true(os.path.ismount(dir_metrics[0]['mountpoint']))
      assert_equals('/path/to/logs', dir_metrics[0]['path'])
    else:
      assert_true(len(dir_metrics) == 0, "We configured 0 dirs: %s" % dir_metrics)

    # Test when the process is not running
    monitor.refresh_pid(None)
    monitor.collect()

    monitor.clear_last_reported_timestamp()
    role_update = monitor.get_role_update().to_avro_dict()

    assert_equals('mgmt-AMON', role_update['rolename'])
    assert_equals('my_mgmt1', role_update['service_name'])
    metric_dict = { }
    for metric_obj in role_update['metrics']:
      metric_dict[metric_obj['id']] = metric_obj['value']

    dir_metrics = role_update['dirUpdates']
    if expect_directory_metrics:
      assert_true(len(dir_metrics) == 1, "We configured 1 dirs: %s" % dir_metrics)
      assert_true(os.path.ismount(dir_metrics[0]['mountpoint']))
      assert_equals('/path/to/logs', dir_metrics[0]['path'])
    else:
      assert_true(len(dir_metrics) == 0, "We configured 0 dirs: %s" % dir_metrics)

    # Test with a collection error
    monitor.refresh_pid(os.getpid(), True)
    monitor.set_throw_error(IOError("BOOM!"))
    monitor.collect()
    monitor.clear_last_reported_timestamp()
    role_update = monitor.get_role_update().to_avro_dict()
    metric_dict = { }
    for metric_obj in role_update['metrics']:
      metric_dict[metric_obj['id']] = metric_obj['value']
    assert_true(metric_dict[schema.TOTAL_CPU_USER] >= 0.0)
    assert_equals(constants.WEB_METRICS_COLLECTION_COMMUNICATION_FAILURE,
                  metric_dict[schema.WEB_METRICS_COLLECTION_STATUS])
    assert_true(schema.WEB_METRICS_COLLECTION_DURATION in metric_dict)

    # Test with some invalid JSON
    monitor.set_throw_error(None)
    monitor.set_string("THIS IS BOGUS JSON!")
    monitor.collect()
    monitor.clear_last_reported_timestamp()
    role_update = monitor.get_role_update().to_avro_dict()
    metric_dict = { }
    for metric_obj in role_update['metrics']:
      metric_dict[metric_obj['id']] = metric_obj['value']
    assert_true(metric_dict[schema.TOTAL_CPU_USER] >= 0.0)
    assert_equals(constants.WEB_METRICS_COLLECTION_PARSE_FAILURE,
                  metric_dict[schema.WEB_METRICS_COLLECTION_STATUS])
    assert_true(schema.WEB_METRICS_COLLECTION_DURATION in metric_dict)

    # Stop it
    monitor.stop()
    assert_equals(0, len(reg.get_registry()))
  finally:
    shutil.rmtree(conf_dir)
    shutil.rmtree(conf_dir2)

