# Copyright (c) 2011-2012 Cloudera, Inc. All rights reserved.

import re

JAVA_MEMORY= re.compile('java.lang:type=Memory$')
EVENT_STATS = re.compile('com.cloudera.cmf.event.publish:type=EventStorePublisherWithRetryMXBean')
EVENT_SERVER = re.compile('com.cloudera.cmf.eventcatcher.server:type=EventCatcherServiceMXBean')
EVENT_PIPELINE = re.compile("com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=tagger-writer")
SMON_ROLE_PIPELINE = re.compile("com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=roleStageTSStore")
HMON_HOST_PIPELINE = re.compile("com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=HMONHostStageReceiver")
AMON_ACTIVITY_TREE_PIPELINE = re.compile("com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=activityTree")
AMON_ACTIVITY_MONITOR_PIPELINE = re.compile("com.cloudera.cmf.event.publish:type=PipelineStageMXBean,stage=activityMonitoring")
AMON_ACTIVITY_AND_ATTEMPTS_PURGER = re.compile("com.cloudera.enterprise:type=PeriodicEnterpriseServiceMXBean,service=ActivityAndAttemptPurgeService")
DATANODE = re.compile('Hadoop:service=DataNode,name=DataNodeActivity-.+-\d+')
DATANODE_DATASET_STATE = re.compile('Hadoop:service=DataNode,name=FSDatasetState-.+')
DATANODE_JVM_METRICS = re.compile('Hadoop:service=DataNode,name=JvmMetrics')
NAMENODE = re.compile('Hadoop:service=NameNode,name=NameNodeActivity')
NAMENODE_RPC = re.compile('Hadoop:service=NameNode,name=RpcActivityForPort\d+')
NAMENODE_NAMESYSTEM = re.compile('Hadoop:service=NameNode,name=FSNamesystem$')
NAMENODE_JVM_METRICS = re.compile('Hadoop:service=NameNode,name=JvmMetrics')
SECONDARYNAMENODE_JVM_METRICS = re.compile('Hadoop:service=SecondaryNameNode,name=JvmMetrics')

class Bean:
  """
  Holds metadata used to represent a bean from which we collect metrics.
  """
  def __init__(self, regex, alias, metric_aliases={}):
      # A regular expression used to identify the bean.
      self.regex = regex
      # The name we'll use internally for the bean.
      self.alias = alias
      # A dictionary of bean metric name to our name for the metric that is used
      # to rename metrics to names we consider more appropriate. If a metric is
      # not present in this list, we will use a standard transform you can find
      # in the metric-tool.
      self.metric_aliases = metric_aliases

_JAVA_MEMORY_BEAN = Bean(re.compile('java.lang:type=Memory$'), 'java_memory')

def _make_jvm_metrics_bean(name):
    return Bean(re.compile('Hadoop:service=%s,name=JvmMetrics' % name), 
                'jvm',
                dict(GcCount='jvm_gc_count',
                     GcTimeMillis='jvm_gc_time_ms',
                     ThreadsNew='jvm_new_threads',
                     ThreadsRunnable='jvm_runnable_threads',
                     ThreadsBlocked='jvm_blocked_threads',
                     ThreadsWaiting='jvm_waiting_threads',
                     ThreadsTimedWaiting='jvm_timed_waiting_threads',
                     ThreadsTerminated='jvm_terminated_threads',
                     MemNonHeapUsedM='jvm_non_heap_used_mb',
                     MemNonHeapCommittedM='jvm_non_heap_committed_mb',
                     MemHeapUsedM='jvm_heap_used_mb',
                     MemHeapCommittedM='jvm_heap_committed_mb'))

def _make_ugi_metrics_bean(name):
    return Bean(re.compile('Hadoop:service=%s,name=UgiMetrics' % name),
                'ugi')

def _make_rpc_metrics_bean(name):
    return Bean(re.compile('Hadoop:service=%s,name=RpcActivityForPort\d+' % name),
                'rpc',
                dict(ReceivedBytes='rpc_received_bytes',
                     SentBytes='rpc_sent_bytes',
                     CallQueueLength='rpc_call_queue_length',
                     NumOpenConnections='rpc_num_open_connections'))

def _make_detailed_rpc_metrics_bean(name):
    return Bean(re.compile('Hadoop:service=%s,name=RpcDetailedActivityForPort\d+' % name),
                'detailed_rpc')

def _make_metrics_control_metrics_bean(name):
    return Bean(re.compile('Hadoop:service=%s,name=MetricsSystem,sub=Control' % name),
                'metrics_control')

def _make_metrics_stats_metrics_bean(name):
    return Bean(re.compile('Hadoop:service=%s,name=MetricsSystem,sub=Stats' % name),
                'metrics_stats',
                dict(DroppedPubAll='metrics_dropped_pub_all',
                     NumActiveSinks='metrics_num_active_sinks',
                     NumActiveSources='metrics_num_active_sources',
                     NumAllSinks='metrics_num_all_sinks',
                     NumAllSources='metrics_num_all_sources',
                     PublishAvgTime='metrics_publish_avg_time',
                     PublishNumOps='metrics_publish_num_ops',
                     SnapshotAvgTime='metrics_snapshot_avg_time',
                     SnapshotNumOps='metrics_snapshot_num_ops'))

BEANS = dict(
    NAMENODE=[
        _JAVA_MEMORY_BEAN,
        _make_jvm_metrics_bean('NameNode'),
        _make_rpc_metrics_bean('NameNode'),
        _make_detailed_rpc_metrics_bean('NameNode'),
        _make_ugi_metrics_bean('NameNode'),
        _make_metrics_control_metrics_bean('NameNode'),
        _make_metrics_stats_metrics_bean('NameNode'),
        Bean(re.compile('Hadoop:service=NameNode,name=NameNodeInfo'),
             'namenode-info'),
        Bean(re.compile('Hadoop:service=NameNode,name=NameNodeActivity'),
             'namenode-activity'),
        Bean(re.compile('Hadoop:service=NameNode,name=FSNamesystem$'),
             'fsnamesystem'),
        Bean(re.compile('Hadoop:service=NameNode,name=FSNamesystemState'),
             'fsnamesystem-state')
        ],
    SECONDARYNAMENODE=[
        _JAVA_MEMORY_BEAN,
        _make_jvm_metrics_bean('SecondaryNameNode'),
        _make_rpc_metrics_bean('SecondaryNameNode'),
        _make_detailed_rpc_metrics_bean('SecondaryNameNode'),
        _make_ugi_metrics_bean('SecondaryNameNode'),
        _make_metrics_control_metrics_bean('SecondaryNameNode'),
        _make_metrics_stats_metrics_bean('SecondaryNameNode'),
        ],
    DATANODE=[
        _JAVA_MEMORY_BEAN,
        _make_jvm_metrics_bean('DataNode'),
        _make_rpc_metrics_bean('DataNode'),
        _make_detailed_rpc_metrics_bean('DataNode'),
        _make_ugi_metrics_bean('DataNode'),
        _make_metrics_control_metrics_bean('DataNode'),
        _make_metrics_stats_metrics_bean('DataNode'),
        Bean(re.compile('Hadoop:service=DataNode,name=DataNodeActivity'),
             'datanode-activity'),
        Bean(re.compile('Hadoop:service=DataNode,name=DataNodeInfo'),
             'datanode-info'),
        Bean(re.compile('Hadoop:service=DataNode,name=FSDatasetState'),
             'fsdatasetstate'),
        ],
    JOURNALNODE=[
        _JAVA_MEMORY_BEAN,
        _make_jvm_metrics_bean('JournalNode'),
        _make_rpc_metrics_bean('JournalNode'),
        _make_detailed_rpc_metrics_bean('JournalNode'),
        _make_ugi_metrics_bean('JournalNode'),
        _make_metrics_control_metrics_bean('JournalNode'),
        _make_metrics_stats_metrics_bean('JournalNode'),
        Bean(re.compile('Hadoop:service=JournalNode,name=Journal-.+'),
             'journal',
             dict(CurrentLagTxns='current_lag_txns_journalnode'))
        ]
    )
